\\ Copyright 2017, 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
default(parisizemax,300*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");
\\ GP-DEFINE  read("../vpar.gp");
\\ GP-DEFINE  read("../vpar-wip.gp");

option_verbose=0;
progress_prev=-1;


\\------------------------------------------------------------------------------

check_equal(numbpart(32), 8349);

tablesize(n) = n + sum(s=1,n, tablesize(n-s));
tablesize=memoize(tablesize);
check_equal(vector(23,n, tablesize(n)), \
            vector(23,n, 2^n-1));

add_seen(n,p) =
{
  my(t=1);
  for(i=1,#p,
     my(s=p[i],
        to=table[t + s-1]);
     n-=p[i];
     if(to,
        t = to;
        ,
        my(node_size = min(n,s));
        \\ print("new node size "node_size" at "nextpos" set ",t + s-1," point there");
        t = table[t + s-1] = nextpos;
        nextpos += node_size);
  );
}
make_table(n) =
{
  table=vector(2^n);
  nextpos=n+1;  \\ after n terms top
  my(v=vecsort(apply(p->Vecrev(p),partitions(n)),,8));
  for(i=1,#v,
     \\ print(v[i]);
     add_seen(n, v[i]));
  table=table[1..nextpos-1];
}
make_table(5)
print("len "nextpos-1" "table[1..nextpos-1])

vector(15,n, make_table(n); #table)
\\ A026905 cumulative numbpart()
A026905(n) = sum(i=1,n, numbpart(i));
{
  my(want=OEIS_samples("A026905"));
  check_equal(vector(#want,n, A026905(n)),
              want,
              "A026905 cumulative numbpart");
}
check_equal(A026905(32), 43819);

check_equal(A026905(20), 2713);
\\ cf n=20    seen array 2663

quit

\\------------------------------------------------------------------------------
\\ A337114 how many distinct unordered orbit sizes, free trees
\\
\\ https://mathpickle.com/project/treefolk-tribes-symmetry-sorting-sequence
\\ (brief)

partitions_min12(n) = select(p->vecmin(p)<=2, partitions(n));
{
  my(n=6);
  my(ps=partitions_min12(n)); print(ps);
  my(m=Map(Mat([ps~,[1..#ps]~])),
     vpar=vpar_free_first(n));
  until((vpar=vpar_free_next(vpar))=='none,
        my(p=vecsort(vpar_free_orbit_sizes(vpar)));
        print("seen "p);
        mapdelete(m,Vecsmall(p)));
  m=Mat(m);
  for(i=1,matsize(m)[1],
     print(Vec(m[i,1])));
  #m
}
quit


a(n)= my(m=Map(),vpar=vpar_free_first(n)); until((vpar=vpar_free_next(vpar))=='none, mapput(m,vecsort(vpar_free_orbit_sizes(vpar)),1)); #m;

for(n=1,20, print1(n" "); print(a(n)));
quit

           mapput(tree_to_index,vpar,i)
           mapput(m,vecsort(vpar_free_orbit_sizes(vpar)),1));
     print1(#m",");

vector(23,n, my(ret=0); forpart(p=n, ret+=(p[1]<=2)); ret) == \
vector(23,n, numbpart(n-1) + #partitions(n-2,[2,n]))

\\ with a part 1: numbpart(n-1)
\\ with a part 2: #partitions(n-2,[2,n])
vector(25,n, numbpart(n-1) + #partitions(n-2,[2,n]))
vector(23,n, numbpart(n) - if(n>=3, #partitions(n,[3,n])))
 1, 2, 2, 4, 6, 9, 13, 19, 26, 37, 50, 68, 91, 122, 159, 210, 272, 352, 451, 578, 732, 929, 1167, 1465, 1828
cf actual A337114:
 1, 1, 1, 2, 3, 6,  9, 15, 19, 32, 36, 56, 70, 103, 122, 175, 210, 298, 349, 486, 569, 773,  912, 1237, 1430

vector(23,n, #partitions(n-2,[2,n]))
A085811 Number of partitions of n including 3, but not 1.



\\------------------------------------------------------------------------------
\\ orbit partitions of n, free forests


{
  \\ free trees
  \\ A337114 = 1,1,1,2,3,6,9,15,19,32,36,56
  \\
  \\ free forests
  \\          1,1,2,4,6,9,15,20,29,37,54,69,96,119,166,
  \\ not in OEIS: 2,4,6,9,15,20,29,37,54,69,96,119,166
  \\ 
  my(forest=1);
  for(n=1,27,
    my(m=Map(),
       vpar=vpar_free_first(n));
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           mapput(m,vecsort(vpar_free_orbit_sizes(vpar)),1));
     print1(#m",");
  );
  quit;
}

\\ 1, 1, 1, 2, 3, 6, 9, 15, 19, 32, 36, 56, 70, 103, 122, 175, 210, 298,
\\ 349, 486, 569,   773, 912, 1237
\\ n=1  a(n) = 1                     
\\ n=2  a(n) = 1                     
\\ n=3  a(n) = 1                     
\\ n=4  a(n) = 2                     
\\ n=5  a(n) = 3                     
\\ n=6  a(n) = 6                     
\\ n=7  a(n) = 9                     
\\ n=8  a(n) = 15                     
\\ n=9  a(n) = 19                     
\\ n=10  a(n) = 32                     
\\ n=11  a(n) = 36                     
\\ n=12  a(n) = 56                     
\\ n=13  a(n) = 70                     
\\ n=14  a(n) = 103                     
\\ n=15  a(n) = 122                     
\\ n=16  a(n) = 175                     
\\ n=17  a(n) = 210                     
\\ n=18  a(n) = 298                     
\\ n=19  a(n) = 349                     
\\ n=20  a(n) = 486                     
\\ n=21  a(n) = 569                     
\\ n=22  a(n) = 773                     
