\\ Copyright 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("memoize.gp");


composition_to_num(vec) =
{
  my(num=0,pos=-1);
  for(i=1,#vec-1,
     pos += vec[i];
     num += 1<<pos);
  num + 1;
}
num_to_composition(num,n) =
{
  num += (1<<(n-1)) - 1;
  my(vec=vectorsmall(hammingweight(num)),
     upto=0);
  while(num,
        my(low=valuation(num,2)+1);
        vec[upto++] = low;
        num >>= low);
  vec;
}

\\ n=4    1 000     [4]
\\ n=4    1 001     [1,3]
\\ n=4    1 011     [1,3,4]  [1,3]
\\ b[pos]=1; pos+=vec[i], but don't want first b[pos]=1
composition_to_num(vec) =
{
  my(b=vector(vecsum(vec)-1), pos=vec[1]);
  for(i=2,#vec, b[pos]=1; pos+=vec[i]);
\\ print(" b="b" from vec="vec" is "fromdigits(b,2)+1);
  fromdigits(b,2) + 1;
}
num_to_composition(num,n) =
{
  num += (1<<(n-1)) - 1;
  my(b=binary(num),
     vec=vector(hammingweight(b)), 
     pos=0);
  for(i=1,#b, ; vec[pos+=b[i]]++);
\\ print(" b="b" to vec="vec"  for num="num-((1<<(n-1)) - 1));
  vec;
}

{
  for(n=4,4,
     for(num=1,2^(n-1),
        my(vec=num_to_composition(num,n));
        my(num_again=composition_to_num(vec));
        printf("n=%d num=%X  %s    num_again=%X\n", n, num, vec, num_again)));
}
{
  for(n=1,8,
     for(num=1,2^(n-1),
        my(vec=num_to_composition(num,n));
        check_equal(vecsum(Vec(vec)), n, "num_to_composition() vecsum");
        check_equal(vecmin(Vec(vec)) >= 1, 1, "num_to_composition() vecmin");
        my(num_again=composition_to_num(vec));
        check_equal(num_again,num, "composition_to_num() again")));
}

Bulgarian_composition_parent(p) =
{
  p=Vec(p);
  for(i=1,#p, p[i]--);
  select(x->x!=0, concat(p,[#p]));
}
make_Bulgarian_compositions(n) =
{
  vector(2^(n-1),v,
         composition_to_num
         (Bulgarian_composition_parent
          (num_to_composition(v,n))));
}

triangular(t) = t*(t+1)/2;


\\-----------------------------------------------------------------------------
\\ Garden of Eden partitions = those with no predecessor, so childless

{
  \\ triangular
  \\ not in OEIS: 1, 19, 423, 15397, 1030865, 
  \\ vector(8,t, 2^(t*(t+1)/2-1) - fibonacci(t*(t+1)/2+1))
  for(t=1,14,
     print1(vpar_num_childless(make_Bulgarian_compositions(triangular(t))),", "));
  quit;
}
{
  \\ all n
  \\ 0,0,1,3,8,19,43,94,201,423,880,1815,3719,7582,15397,31171,
  \\ A008466 2^(n-1) - Fibonacci(n+1) per Hopkins and Jones
  for(n=1,30,
     print1(vpar_num_childless(make_Bulgarian_compositions(n)),","));
  quit;
}


\\-----------------------------------------------------------------------------
\\ Compositions Ranking and Unranking

{
  for(n=1,4,
     for(num=0,2^(n-1)-1,
        my(vec=num_to_composition(num,n));
        my(num_again=composition_to_num(vec));
        printf("n=%d num=%X  %s  num_again=%X\n", n, num, vec, num_again)));
}
