\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

option_verbose=0;

read("OEIS-data.gp");
OEIS_check_verbose = 1;
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("a348959-childless-terminal-Wiener.gp");


\\-----------------------------------------------------------------------------

\\ A288469
Matula_unplant(n) = while(isprime(n),n=primepi(n)); n;
A288469 = Matula_unplant;
{
  my(want=OEIS_data("A288469"));  \\ OFFSET=1
  check_equal(vector(#want,n, A288469(n)),
              want,
              "A288469 Matula_unplant() delete root path");
}

A196055(n) = vpar_Wiener_index(vpar_from_primecode(n),1);
{
  my(want=OEIS_data("A196055"));  \\ OFFSET=1
  check_equal(vector(#want,n, A196055(n)),
              want,
              "A196055 free terminal Wiener");
}

\\ A196048 external path length, sum depths childless
A196048_external(n) =
{
  my(vpar=vpar_from_primecode(n));
  vecsum(vecextract(vpar_depths_vector(vpar),
                    vpar_childless_vertices(vpar)));
}
{
  my(want=OEIS_data("A196048"));  \\ OFFSET=1
  check_equal(vector(#want,n, A196048_external(n)),
              want,
              "A196048 sum depths childless");
}

\\ A109129 num non-root childless vertices of tree primecode n
num_childless_with_root(n) = \
  if(n==1,1, my(f=factor(n)); [self()(primepi(p)) |p<-f[,1]] * f[,2]);
A109129_num_childless(n) = if(n==1,0, num_childless_with_root(n));
{
  my(want=OEIS_data("A109129"));    \\ OFFSET=1
  check_equal(vector(#want,n, A109129_num_childless(n)),
              want,
              "A109129() num non-root childless in primecode");
}

\\-----------------------------------------------------------------------------

{
  my(want=OEIS_data("A348959"));
  check_equal(vector(#want,n, a(n)),
              want,
              "a() vs samples");

  if(0,
     \\ DATA
     for(n=1,73,print1(a(n),",");if(n==28||n==51,print()));
     print();
     quit);

  if(0,
     system("rm -f /tmp/new/b348959.txt");
     for(n=1,10000,
        write("/tmp/new/b348959.txt", Str(n," ",a(n))));
     system("ls -l /tmp/new/b348959.txt");
     quit);
  \\ ~/OEIS/b348959.txt
  if(0,
     my(want=OEIS_data("A348959",'bfile));
     print("b348959.txt length ",#want);
     check_equal(vector(#want,n, a(n)),
                 want,
                 "A348959() vs ~/OEIS/b348959.txt"));
}

\\-----------------------------------------------------------------------------

\\ Return a vector of the prime factors of n, with multiplicity, in
\\ ascending order (A027746).
factor_to_primes(n) =
{
  if(n==1,[],
     my(f=factor(n));
     concat([vector(f[i,2],j,f[i,1]) |i<-[1..matsize(f)[1]]]));
}
A027746_row = factor_to_primes;
{
  my(want=OEIS_data("A027746"),
     got=[]);
  for(n=1,oo, if(#got>=#want,break);
     got=concat(got, if(n==1,[1], A027746_row(n))));
  got=got[1..#want];
  check_equal(got,want, "A027746 factor_to_primes()");
}

factor_to_primepis(n) = apply(primepi,factor_to_primes(n));
A112798_row = factor_to_primepis;
{
  my(want=OEIS_data("A112798"),
     got=[]);
  for(n=2,oo, if(#got>=#want,break);
     got=concat(got, if(n==1,[1], A112798_row(n))));
  got=got[1..#want];
  check_equal(got,want, "A112798 factor_to_primepis()");
}


\\-----------------------------------------------------------------------------

W_and_terminals(n) =  \\ return 2-element vector [W,t]
{
  if(n<=1, [0,1],
     my(f=factor(n),wt);
     [ (wt=self()(primepi(p))) + [wt[2]*('total-wt[2]), 0]
      | p<-f[,1] ]
     * f[,2]);
}

{
  check_equal(vector(200,n, W_and_terminals(n)[2]),
              vector(200,n, num_childless_with_root(n)));

  check_equal(vector(200,n, polcoeff(W_and_terminals(n)[1],1)),
              vector(200,n, A196048_external(n)),
              "poly 'total coeff is E sum depths");
}

num_childless_vector(vpar) =
{
  my(vec=vector(#vpar,i,-1),
     seq=vpar_INTERNAL_seq_upwards(vpar));
  for(i=1,#seq,
     my(v=seq[i], p=vpar[v]);
     vec[v]=abs(vec[v]);
     if(p, vec[p]=max(0,vec[p])+vec[v]));
  vec;
}
check_equal(num_childless_vector([0,1,2,3]), [1,1,1,1]);
check_equal(num_childless_vector([0,1,1,3]), [2,1,1,1]);
check_equal(num_childless_vector([0]), [1]);

sum_squares_num_childless_except_root_of_vpar(vpar) =
{
  my(vec=num_childless_vector(vpar));
  vecsum(apply(sqr,vec[^vpar_root(vpar)]));
}
check_equal(sum_squares_num_childless_except_root_of_vpar([0,1,1,3]), 1+1+1);
{
  check_equal(W_and_terminals(1), [0, 1]);
  check_equal(sum_squares_num_childless_except_root_of_vpar([0]), 0);

  check_equal(vector(500,n, -polcoeff(W_and_terminals(n)[1],0)),
              vector(500,n, sum_squares_num_childless_except_root_of_vpar(vpar_from_primecode(n))),
              "poly constant coeff is sum squares" );
}

\\ print(vector(20,n, -polcoeff(W_and_terminals(n)[1],0)));
\\ print(vector(20,n, sum_squares_num_childless_except_root_of_vpar(vpar_from_primecode(n)))); quit;
\\ not in OEIS: 2, 2, 3, 3, 6, 3, 4, 4, 4, 4, 7, 7, 5, 4, 10, 5, 12, 5, 8
\\
\\ print(vector(20,n, vecsum(apply(sqr,num_childless_vector(vpar_from_primecode(n)))))); quit;
\\ not in OEIS: 1, 2, 3, 6, 4, 7, 10, 12, 8, 8, 5, 13, 11, 16, 9, 20, 14, 14, 21, 14
\\
\\ print(vector(20,n, vecsum(apply(sqr,vpar_subtree_sizes_vector(vpar_from_primecode(n)))))); quit;
\\ not in OEIS: 1, 5, 14, 11, 30, 22, 27, 19, 35, 40, 55, 32, 47, 37, 55, 29, 52, 47, 44, 52

\\-----------------------------------------------------------------------------

{
  \\ differs from A196055
  check_equal(vector(1000,n, a(n) != A196055(n)),
              vector(1000,n, isprime(n)),
              "a() differs A196055() when n prime");
  check_equal(vector(1000,n,a(n)),
              vector(1000,n,A196055(n) - if(isprime(n), A196048_external(n))),
              "a() vs A196055() and isprime()");
  check_equal(vector(1000,n,a(n)),
              vector(1000,n,A196055(A288469(n))),
              "a() vs A196055() and unplant A288469()");

}
{
  \\ formula a(n) = Sum_{j=1..k} a(pi[j]) + C(pi[j])*(C(n)-C(pi[j]))
  my(C(n) = A109129_num_childless(n));
  my(E(n) = A196048_external(n));

  check_equal(factor_to_primepis(4), [1,1]);
  check_equal(a(1), 0);
  check_equal(C(2), 1);

  check_equal(factor_to_primepis(6), [1,2]);
  check_equal(a(1), 0);
  check_equal(a(2), 0, "a(2)");
  check_equal(C(2), 1);
  check_equal(C(6), 2);

  check_equal(vector(20,n,
                     my(p=factor_to_primes(n),
                        k=#p);
                     sum(j=1,k, a(primepi(p[j])) + E(p[j])*(C(n)-C(p[j]))) ),
              vector(20,n, a(n)),
              "a() by formula()");
}

\\-----------------------------------------------------------------------------
\\ Methods Cross-Checked

Matula_childless_TW_triplet_WEV(n) =
{
  \\ print("Matula_childless_TW_triplet_WEV() ",n);
  if(n==1,[0,0,1],
     my(f=factor(n),W=0,E=0,V=0);
     for(i=1,matsize(f)[1],
        my(v=self()(primepi(f[i,1])));
        \\ print("recursed v="v" onto p="f[i,2]" at "W" "E" "V"  acc "v);
        \\ for(j=1,f[i,2],
        \\    W += v[1]      \\ W within child
        \\        + E*v[3]   \\ each E up to root
        \\        + (v[2]+v[3])*V;
        \\    E += v[2] + v[3];
        \\    V += v[3]);
        v[2] += v[3];
        \\ print("  incr ",v);
        W += v[1]*f[i,2];     \\ within children
        \\ print(E " " E * v[3]*f[i,2]); \\ E up to root
        W += E * v[3]*f[i,2]; \\ E up to root
        W += v[2]*V*f[i,2];   \\ new up to root
        \\ print("  mutual ",v[2]*v[3]*(f[i,2]-1)*f[i,2]);
        W += v[2]*v[3]*(f[i,2]-1)*f[i,2];  \\ mutuals
        E += v[2] *f[i,2];
        V += v[3] *f[i,2];
     );
     [W,E,V]);
}
{
  check_equal(Matula_childless_TW_triplet_WEV(1), [0,0,1],
              "WEV of n=1 singleton");
  check_equal(Matula_childless_TW_triplet_WEV(2), [0,1,1],
              "WEV of n=2 path-2");
  \\      R
  \\     / \
  \\    A   B
  check_equal(Matula_childless_TW_triplet_WEV(4), [2,2,2],
              "WEV of n=4 2-star");
}

childless_TW_by_WEV(n) = Matula_childless_TW_triplet_WEV(n)[1];
{
  my(len=20000,
     by_a=vector(len,n, a(n)),
     by_WEV=vector(len,n, childless_TW_by_WEV(n)),
     by_vpar=vector(len,n, vpar_Wiener_index(vpar_from_primecode(n),2)));
  check_equal(by_a,
              by_vpar,
              "a() vs vpar_Wiener_index()");
  check_equal(by_a,
              by_WEV,
              "a() vs childless_TW_by_WEV()");
}

\\-----------------------------------------------------------------------------

\\ test_terminal_Wiener_index_childless_WV(n) =
\\ {
\\   if(n<=1,[0,1],
\\      my(f=factor(n),wv);
\\      [(wv=self()(primepi(p)))+[wv[2]*('t-wv[2]),0] |p<-f[,1]] * f[,2]);
\\ }
\\ test_terminal_Wiener_index_childless(n) =
\\ {
\\   my(wv=test_terminal_Wiener_index_childless_WV(n));
\\   subst(wv[1],'t,wv[2]);
\\ }

  \\ check_equal(test_terminal_Wiener_index_childless_W_edges_C(1), [0,0,1],
  \\             "WEC of n=1 singleton");
  \\ check_equal(test_terminal_Wiener_index_childless_W_edges_C(2), [0,1,1],
  \\             "WEC of n=2 path-2");
  \\ check_equal(test_terminal_Wiener_index_childless_W_edges_C(3), [0,2,1],
  \\             "WEC of n=3 path-3");

  \\ check_equal(test_terminal_Wiener_index_childless_W_edges_C(4), [2,2,2],
  \\             "WEC of n=4 star-2");
  \\ check_equal(test_terminal_Wiener_index_childless_W_edges_C(8), [6,3,3],
  \\             "WEC of n=4 star-3");



\\-----------------------------------------------------------------------------
\\ UNUSED:

\\ Formula in Sequence
\\
\\ The formula given in the sequence is expressed just on the prime factors
\\ of n.  The following a_by_formula() is that method.
\\
\\ But this is a naive implementation only for a check and compare.  In
\\ particular there are repeat primepi()s and factor()s, and C and E could
\\ easily be calculated together.
\\
\\ C(n) is implemented with C(1)=1 which is a singleton root reckoned as 1
\\ chlidless vertex.  This is convenient for its recursion, and makes no
\\ difference in a_by_formula().  Its C(p[j]) is prime p[j] never 1, and
\\ C(n) for n=1 is an empty factorization p==[] so the empty sum.

\\ Number of childless vertices in tree n.
\\ C(n0 = A109129(n), except that C(1)=1 unlike A109129(1)=0.
C(n) =
{
  if(n==1,1,
     my(f=factor(n));
     [self()(primepi(p)) |p<-f[,1]] * f[,2]);
}

\\ Sum of depths (distance to the root) of all childless vertices in tree n.
\\ E(n) = A196048(n)
E(n, depth=0) =
{
  my(f=factor(n));
  if(n==1,depth,
     [self()(primepi(p),depth+1) |p<-f[,1]] * f[,2]);
}

a_by_formula(n) =
{
  my(p=factor_to_primes(n));
  sum(j=1,#p, a(primepi(p[j])) + E(p[j])*(C(n)-C(p[j])));
}

{
  my(count=0);
  for(n=1,500,
     a(n) == a_by_formula(n)  || error();
     count++);
  print("(",count," checks of a() = a_by_formula()");
}

\\-----------------------------------------------------------------------------
check_equal(f,'f);
check_equal(b,'b);
check_equal(n,'n);
check_equal(t,'t);
check_equal(W,'W);
check_equal(wt,'wt);

print("end");
