\\ Copyright 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Tests for a354322-Matula-subtree-rows.gp, calculating fows of A354322,
\\ the distinct subtrees occurring in Matula-Goebel tree n.


default(recover,0);
default(strictargs,1);
default(parisizemax,1800*10^6);  \\ for big search

read("a354322-Matula-subtree-rows.gp");
print("------------\n");

option_verbose=0;
option_progress=1;

read("memoize.gp");
read("OEIS-data");
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");


\\ not in OEIS: 1, 2, 3, 5, 11, 31, 127, 527, 1829, 7493, 35179, 196393, 1266983, 6297041
\\ next > 16 million


\\-----------------------------------------------------------------------------

{
  my(v=Vecsmall([11,12,13,14,15]));
  check_equal(v[2..4], Vecsmall([12, 13, 14]));
  check_equal(type(v[2..4]), "t_VECSMALL");

  check_equal(type(vecextract(v,"2..4")), "t_VECSMALL");

  \\ v[3..5] = [103,104,105];
  \\ check_equal(v,0);
}

\\-----------------------------------------------------------------------------


{
  \\ smallest primecode with n distinct subtrees
  \\ path down is an upper bound
  my(a(n) = for(primecode=1,oo,
               if(#row(primecode)==n,
                  return(primecode))));
  if(0,
     print(vector(12,n,a(n)));
     quit);
  \\ 1, 2, 3, 5, 11, 31, 127, 527, 1829, 7493, 35179, 196393

  \\ print(vector(9,n, a(n)));
  \\ check_equal(vector(9,n, a(n) == A007097_primeth(n-1)),
  \\             vector(9,n, n <= 7));              
  
  check_equal(vpar_from_primecode(11),
              [0, 1, 2, 3, 4]);
  check_equal(vpar_from_primecode(31),
              [0, 1, 2, 3, 4, 5]);
  check_equal(vpar_from_primecode(127),
              [0, 1, 2, 3, 4, 5, 6]);

  \\ n=8 distint, 10 vertices
  \\          527
  \\         /   \
  \\       11     7  
  \\        |     |  
  \\        5     4  
  \\        |    / \ 
  \\        3   1   1
  \\        |
  \\        2
  \\        |
  \\        1
  my(vpar=[0,1,2,3,3, 1,6,7,8,9],
     primecode=527);
  check_equal(#vpar, 10);
  check_equal(primecode, vpar_to_primecode(vpar));
  check_equal(vector(#vpar,v, vpar_to_primecode(vpar_subtree(vpar,v))),
              [527,7,4,1,1, 11,5,3,2,1]);

  \\ n=9 distint, 11 vertices
  \\          1829
  \\         /    \
  \\       11     17
  \\        |      |
  \\        5      7  
  \\        |      |  
  \\        3      4  
  \\        |     / \ 
  \\        2    1   1
  \\        |
  \\        1
  my(vpar=[0,1,2,3,4,5, 1,7,8,9,9],
     primecode=1829);
  check_equal(#vpar, 11);
  check_equal(primecode, vpar_to_primecode(vpar));
  check_equal(vector(#vpar,v, vpar_to_primecode(vpar_subtree(vpar,v))),
              [1829,11,5,3,2,1, 17,7,4,1,1]);


  check_equal(vpar_from_primecode(196393),
              [0,1,2,3,4,5,5, 1,8,9,10,11,12,13]);

  check_equal(vpar_from_primecode(6297041),
              [0,1,2,3,4,5,6, 1,8,9,10,10,12,1,14,15,16,17,17]);

  \\ vpar_to_primecode([0,1,2,3,4,5,6, 1,8,9,10,10,12,1,14,15,16,17,17, 7]) == 35 154 347
}

\\-----------------------------------------------------------------------------

limit = 100 * 10^6;
pos_size  = 16 * 10^6;
rows_size = 16 * 10^6;
cache_pos  = vectorsmall(pos_size);
cache_end  = cache_pos;
cache_rows = vectorsmall(rows_size);
cache_rows_used = 0;

cache_pos[1] = 1;
cache_end[1] = 1;

\\ cache_pos[n] is the location within cache_rows[] of row for tree n.
\\ cache_rows[] omits the fixed first 1 and last n from each row.
\\     pos     = cache_pos[n]
\\     nextpos = cache_pos[n+1]
\\     row(n) = 1, cache_rows[pos .. nextpos-1], n


\\ Return a vector of the distinct subtrees of tree n, so row n of A354322.
\\ f = factor(n) if known, or f=0 if not yet known.
\\
row_cached(n,f=0) =
{
  if(n==1,
     [],

     n <= #cache_pos && cache_pos[n],
     cache_rows[cache_pos[n] .. cache_end[n]],

     if(!f, f=factor(n));
     f=f[,1];  \\ primes
     my(ret=[]);
     for(i=1,#f,
        my(this=row_cached(primepi(f[i])));
        ret=setunion(ret,Vec(this));
     \\ print("  this p="f[i]" pi="primepi(f[i])," "this" union to "ret);
     );
     ret = concat(ret,n);

     cache_pos[n]=cache_rows_used+1;
     cache_end[n]=cache_rows_used + #ret;
     for(i=1,#ret,
        cache_rows[cache_rows_used++] = ret[i]);
     \\ print("  n="n" ret "ret);
     ret);
}

{
  check_equal(row_cached(2), [2], "2 first");
  check_equal(row_cached(2), Vecsmall([2]), "2 cached");
}

{
  my(target=2);
  forfactored(ff=2,limit,
     if(vecmax(ff[2][,2]) > 1,
        \\ duplicate at the root, not smallest
        next);

     my(this=row_cached(ff[1],ff[2]));
     if(ff[1] < 1000,
        check_equal(Vec(this),
                    row(ff[1])[^1],
                    Str("row() cached "ff[1])));

     if(#this==target,
        print("n="target"  ",ff[1],"   (used ",cache_rows_used);
        target++);
  );
}

\\-----------------------------------------------------------------------------
print("end");
