#!/usr/bin/perl -w

# Copyright 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  See the file COPYING.  If not, see
# <http://www.gnu.org/licenses/>.

use 5.010;
use strict;

{
  # near perfect matching states 4
  my @vecsmall = (
                      # unm   matched   
                      # w  wo w  wo     
  2,  1,  4,  3,      # 0, 1, 0, 0      
  3,  2,  3,  0,      # 1, 0, 0, 1      
  0,  3,  0,  0,      # 0, 0, 1, 0      
  3,  4,  0,  0       # 1, 0, 0, 0      
              );
  my $n = sqrt(scalar(@vecsmall));
  my @table;
  push @table, [ (undef)x($n+1) ];
  foreach my $i (1 .. $n) {
    push @table, [ undef, @vecsmall[($i-1)*$n ..($i-1)*$n + $n-1] ];
  }
  my $accepting_bits = 14;
  my @accepting_list;
  foreach my $i (1 .. $n) {
    if ($accepting_bits & (1<<$i)) { push @accepting_list, $i }
  }
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@table,
                                   starting => 1,
                                   accepting_list => \@accepting_list);
  $f = $f->as_nfa;
  $f = bless $f, 'FLAT::NFA';
  foreach my $from_state ($f->get_states) {
    $f->remove_transition($from_state,0);
  }
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;

  $f = $f->MyFLAT::minimize (verbose => 1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # almost equimatchable states 14
  my @vecsmall = (
  2,  3,  4,  5,  6,  7,  8,  9, 10, 11,  0,
  2,  6,  2,  8,  6,  8,  8,  0,  8,  0,  0,
  6,  3,  6,  9,  6,  9,  0,  9,  0,  0,  0,
  2,  6,  4,  8,  6, 10,  8,  0, 10,  0,  0,
  8,  9,  8, 11,  0, 11,  0,  0,  0,  0,  0,
  6,  6,  6,  0,  6,  0,  0,  0,  0,  0,  0,
  8,  9, 10, 11,  0,  0,  0,  0,  0,  0,  0,
  8,  0,  8,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  9,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  8,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
              );
  my $n = sqrt(scalar(@vecsmall));
  my @table;
  push @table, [ (undef)x($n+1) ];
  foreach my $i (1 .. $n) {
    push @table, [ undef, @vecsmall[($i-1)*$n ..($i-1)*$n + $n-1] ];
  }
  my $accepting_bits = 3024;
  my @accepting_list;
  foreach my $i (1 .. $n) {
    if ($accepting_bits & (1<<$i)) { push @accepting_list, $i }
  }
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@table,
                                   starting => 1,
                                   accepting_list => \@accepting_list);
  $f = $f->as_nfa;
  $f = bless $f, 'FLAT::NFA';
  foreach my $from_state ($f->get_states) {
    $f->remove_transition($from_state,0);
  }
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;

  $f = $f->MyFLAT::minimize (verbose => 1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # edge stable equimatchable states 9
  my @vecsmall = (
  2,  3,  0,  5,  6,  0,  3,  9,  0,
  4,  0,  0,  0,  7,  0,  0,  7,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,
  4,  0,  0,  0,  4,  0,  0,  4,  0,
  0,  0,  0,  8,  0,  0,  0,  0,  0,
  7,  0,  0,  0,  6,  0,  0,  6,  0,
  4,  0,  0,  0,  7,  0,  0,  7,  0,
  0,  0,  0,  8,  0,  0,  0,  0,  0,
  7,  0,  0,  0,  6,  0,  0,  9,  0
              );
  my $n = sqrt(scalar(@vecsmall));
  my @table;
  push @table, [ (undef)x($n+1) ];
  foreach my $i (1 .. $n) {
    push @table, [ undef, @vecsmall[($i-1)*$n ..($i-1)*$n + $n-1] ];
  }
  my $accepting_bits = 318;
  my @accepting_list;
  foreach my $i (1 .. $n) {
    if ($accepting_bits & (1<<$i)) { push @accepting_list, $i }
  }
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@table,
                                   starting => 1,
                                   accepting_list => \@accepting_list);
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  exit 0;
}

{
  # equimatchable 12 states, with extras
  # WRONG

  # 4 = path-5 is equimatchable, but under anything goes to not
  my @table = (
[undef, undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,],
[undef, 2,3,4,5,12,12,12,12,12,12,12,],
[undef, 2,12,2,12,12,12,12,12,12,12,12,],
[undef, 12,3,12,6,12,12,12,12,12,12,12,],
[undef, 2,12,4,12,12,12,12,12,12,12,12,],
[undef, 12,6,12,7,12,12,12,12,12,12,12,],
[undef, 12,6,12,8,12,12,12,12,12,12,12,],
[undef, 12,8,12,9,12,12,12,12,12,12,12,],
[undef, 12,8,12,10,12,12,12,12,12,12,12,],
[undef, 12,10,12,11,12,12,12,12,12,12,12,],
[undef, 12,10,12,12,12,12,12,12,12,12,12,],
[undef, 12,12,12,12,12,12,12,12,12,12,12,],
[undef, 11,11,11,11,11,11,11,11,11,11,11,],
              );
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@table,
                                   starting => 1,
                                   accepting_list => [ 
4,6,7,8,9,10,11,12
 ]);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  exit 0;
}

{
  # equimatchable states 5

  my @table = (
[undef, undef,undef,undef,undef,undef],
[undef, 2, 3, 4, 5, 0 ],
[undef, 2, 0, 2, 0, 0 ],
[undef, 0, 3, 0, 0, 0 ],
[undef, 2, 0, 4, 0, 0 ],
[undef, 0, 0, 0, 0, 0 ],
              );
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@table,
                                   starting => 1,
                                   accepting_list => [ 1,2,3,5 ]);
  # $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;
  exit 0;
}



