\\ Copyright 2018, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(parisizemax,500*10^6);
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("../examples/integral-tree-diameter.gp");
print("\n-------------");

is_integral_poly(poly) =
{
  my(m=factor(poly));
  for(i=1,matsize(m)[1],
     if(poldegree(m[i,1])>1, return(0)));
  1;
}
is_integral_spectrum(vpar) = \
  is_integral_poly(charpoly(vpar_adjacency_matrix(vpar)));



\\------------------------------------------------------------------------------
\\ free trees integral except biggest eigenvalue
\\
\\ n = 0     3
\\     0,0,0,1,2,2,5,9,21,45,104,234,550,1300,3157,7739,19317,
\\ not in OEIS: 5,9,21,45,104,234,550,1300,3157,7739
\\
\\ n=3
\\   [0, 1, 1] path-3  x^3 - 2*x root [0] rest x^2 - 2 is sqrt(2)
\\   (root 0 means no perfect matching)
\\
\\ n=4
\\   [0, 1, 2, 1] x^4 - 3*x^2 + 1 []     x^4 - 3*x^2 + 1
\\   [0, 1, 1, 1] x^4 - 3*x^2     [0, 0] x^2 - 3

poly_lowest_nonzero(p) = polcoeff(p,valuation(p,variable(p)));
check_equal(poly_lowest_nonzero(x^2*(x-1)^2*(x^2-x-1)), -1)

\\ return [ q, [root,root,...] ]
poly_integer_roots(p,limit=abs(poly_lowest_nonzero(p))) =
{
  my(x=variable(p), q,r, l=List([]));
  for(root=-limit,limit,
     my(d=x-root);
     while([q,r]=divrem(p,d); !r,
           listput(l,root);
           p=q));
  [p, Vec(l)];
}
check_equal(poly_integer_roots(x^2*(x-1)^2*(x^2-x-1)), [x^2-x-1, [0,0,1,1]])

{
  my(forest=0, q,roots);
  for(n=0,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(p=vpar_adjacency_poly(vpar));
           \\ p /= x^valuation(p,x);
           [q,roots] = poly_integer_roots(p);
\\ print(roots);
           roots==-Vecrev(roots) || error();  \\ pairs +root and -root
           if(#roots + if(#roots,polsturm(q,[roots[1],roots[#roots]])) < poldegree(p),
              count++;
              if(n==4,
                 print(vpar" "p" "roots" "q);
                 );
             ));
     print1(count","));
  quit;
}

quit



\\------------------------------------------------------------------------------
\\ free trees has n distinct roots
\\ 1,1,1,1,1,2,2,5,4,18,11,62,37,250,116,1041,465,4452,
\\ not in OEIS: 4,18,11,62,37,250,116,1041,
\\
\\ free trees has < n roots
\\ 0,0,0,0,1,1,4,6,19,29,95,173,514,1051,3043,6700,18855,
\\ not in OEIS: 4,6,19,29,95,173,514,1051,3043,

\\ free forests has n distinct roots
\\ 1,1,2,1,4,3,10,5,32,15,101,49,387,149,
\\ not in OEIS: 4,3,10,5,32,15,101,49,387,149,
\\
\\ free forests has < n distinct roots
\\ 0,1,1,5,6,17,27,71,121,314,609,1552,3271,8450,
\\ not in OEIS: 5,6,17,27,71,121,314,609,1552,3271,8450,

\\ matchpoly always constant term 1
\\ poldegree = matchnum
\\ so perfect or near perfect matching so low term at most x
\\ (not x^2 duplicating 0)

{
  my(forest=0);
  for(n=0,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(p=vpar_adjacency_poly(vpar));
           if(polsturm(p)!=n,
              count++;
              if(n<=5,
                 print(vpar" "vpar_matchnum(vpar)" "p);
                 );
             ));
     print1(count","));
  quit;
}



\\------------------------------------------------------------------------------
\\ free trees with a repeated non-integer root
\\
\\ 0,0,0,0,0,0,0,0,0,4,4,8,21,86,186,
\\ not in OEIS: 4,8,21,86,186,
\\
\\ n=10  sqrt2 repeat
\\ [x, 2;  x^2 - 2, 2;  x^4 - 5*x^2 + 3, 1]
\\ [x, 2;  x^2 - 2, 2;  x^4 - 5*x^2 + 2, 1]
\\ [x, 2;  x^2 - 2, 2;  x^4 - 5*x^2 + 1, 1]
\\ [x, 4;  x^2 - 5, 1;  x^2 - 2, 2]
\\
\\ n=11  sqrt2 repeat
\\ [x - 1, 1;  x, 3; x + 1, 1; x^2 - 5, 1; x^2 - 2, 2]
\\ [x, 3;   x^2 - 2, 2; x^4 - 6*x^2 + 4, 1]
\\ [x, 3;   x^2 - 2, 2; x^4 - 6*x^2 + 3, 1]
\\ [x, 3;   x^2 - 2, 2; x^4 - 6*x^2 + 2, 1]

\\ a,b integers range.
\\ Bisection search for pairs of roots between consecutive integers.
is_repnonint(p) =
{
  my(v=vecsort(polrootsreal(p)));
  for(i=1,#v-1,
     if(abs(v[i]-v[i+1])<1e-10 && floor(v[i])!=v[i],
        return(1)));
  0;
}

{
  default(realprecision,100);
  my(forest=0);
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(p=vpar_adjacency_poly(vpar));
           if(is_repnonint(p),
              count++;
              if(n==13,
                 print(factor(p)));
             ));
     print1(count","));
  quit;
}


\\------------------------------------------------------------------------------
\\ free forests roots without integer between
\\
\\ polsturm(poly,[a,b]) closed interval

check_equal(polsturm(x^2-4), 2);
check_equal(polsturm(x^2-4, [-1,1]), 0);
check_equal(polsturm((x^2-4)*x, [-1,1]), 1);
check_equal(polsturm((x^2-4)*x^3, [-1,1]), 1);
check_equal(polsturm(x^6, [-1,1]), 1);   \\ is without repetition

pol_is_root(p,x) = subst(p,variable(p),x)==0;
check_equal(pol_is_root(x^2-4,2), 1);
check_equal(pol_is_root(x^2-4,0), 0);
check_equal(pol_is_root(x^2-4,-2), 1);
check_equal(pol_is_root(x^2-4,-3), 0);

\\ Return the number of distinct real roots between a,b, not including a,b
\\ themselves.
polsturm_strictly_between(p,a,b) = \
  polsturm(p,[a,b]) - pol_is_root(p,a) - (a!=b && pol_is_root(p,b));

check_equal(polsturm_strictly_between(x^2-4,-3,3), 2);
check_equal(polsturm_strictly_between((x^2-4)*x^3,-3,3), 3);
check_equal(polsturm_strictly_between((x^2-4)*x^3,-3,-1), 1);
check_equal(polsturm_strictly_between((x^2-4)*x^3,-2,2), 1);
check_equal(polsturm_strictly_between((x^2-4)*x^3,-2,3), 2);

\\ Return the number of distinct real roots between a,b, not including a,b
\\ themselves.
polsturm_ge2_strictly_between(p,a,b) =
{
  my(s=polsturm(p,[a,b]));
  if(s>=4, return(1));
  s -= pol_is_root(p,a);
  if(s>=3, return(1));
  s - (a!=b && pol_is_root(p,b)) >= 2;
}

check_equal(polsturm_ge2_strictly_between((x^2-4)*x^3,-3,3), 1);
check_equal(polsturm_ge2_strictly_between((x^2-4)*x^3,-2,3), 1);
check_equal(polsturm_ge2_strictly_between((x^2-4)*x^3,-2,2), 0);

\\ a,b integers range.
\\ Bisection search for pairs of roots between consecutive integers.
is_roots_close_range(p,a,b) =
{
  a < b || error();
  if(a+1==b, return(polsturm_ge2_strictly_between(p,a,a+1)));
  my(m=(a+b)>>1);
  (a < m && m < b) || error();
  (polsturm(p,[a,m]) >= 2 && is_roots_close_range(p,a,m))
  ||
  (polsturm(p,[m,b]) >= 2 && is_roots_close_range(p,m,b));
}

\\ Return 1 if polynomial p has two real roots without an integer between,
\\ so some a,b with n < a < b < n+1 where n an integer.
is_roots_close(p) =
{
  my(hi=abs(polcoeff(p,valuation(p,variable(p))))+1);
  is_roots_close_range(p,-hi,hi);  
}

is_all_roots_ok_delete(vpar) =
{
  for(v=1,#vpar,
     my(deleted_vpar=vpar_delete_vertex(vpar,v));
     if(is_roots_close(vpar_adjacency_poly(deleted_vpar)),
        return(0)));
  1;
}

{
  \\ non-integral trees yet all T\v ok
  \\ 0,0,1,2,2,5,4,10,19,34,46,60,59,68,71,
  \\ not in OEIS: 10,19,34,46,60,59,68,71,

  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next(vpar))=='none,  \\ trees
           my(poly=vpar_adjacency_poly(vpar));
           if(is_integral_poly(poly), next);    \\ non-integral

           if(is_all_roots_ok_delete(vpar),
              count++;
              \\ print("n="n);
              \\ print("  vpar="vpar" "polrootsreal(poly));
              \\ for(v=1,#vpar,
              \\    my(deleted_vpar=vpar_delete_vertex(vpar,v),
              \\       poly=vpar_adjacency_poly(deleted_vpar));
              \\    print("  "deleted_vpar" "polrootsreal(poly)));
             ));
     print1(count,","));
  quit;
}


{
  \\ num trees, two roots without integer between
  \\ 0,0,0,0,0,2,3,4,13,53,168,469,1177,3001,7530,19049,48199,
  \\ not in OEIS: 13,53,168,469,1177,3001,7530,
  \\
  \\ num forests, two roots without integer between
  \\ 0,0,0,0,0,2,7,16,40,118,356,1021,2743,7169,
  \\ not in OEIS: 7,16,40,118,356,1021,2743,
  \\
  \\ a(6)=2 two trees n=6,
  \\   *--*--*--*--*--*     path-6
  \\   poly (x^3 - x^2 - 2*x + 1)*(x^3 + x^2 - 2*x - 1)
  \\   roots a = 1.246... = A255249  \ no integer between
  \\         b = 1.801... = A160389  / 
  \\
  \\   *-\
  \\      *--*--*--*
  \\   *-/
  \\   poly (x^4 - 5*x^2 + 5)*x^2
  \\   roots a = 1.175... = A182007  \ no integer between
  \\         b = 1.902... = A188593  / 
  \\
  \\ vpar_adjacency_matrix(vpar_make_path(6))

  \\ num trees, always integer between roots
  \\ 1,1,1,2,3,4,8,19,34,53,67,82,124,158,211,
  \\ not in OEIS: 3,4,8,19,34,53,67,82,124,158,211

  \\ num forests, always integer between roots
  \\ 1,2,3,6,10,18,30,60,113,211,354,580,915,1430,
  \\ not in OEIS: 3,6,10,18,30,60,113,211,354,580,915,1430



  \\ ~/OEIS/b160389.txt
  \\ g=OEIS_bfile_gf("A160389");
  \\ rlo = subst(g,        'x,1/10) * 10;
  \\ rhi = subst(g+x^20000,'x,1/10) * 10;
  \\ rlo*1.0
  \\ rhi*1.0
  \\ subst(x^3 - x^2 - 2*x + 1, x, rlo) < 0
  \\ subst(x^3 - x^2 - 2*x + 1, x, rhi) > 0

  \\ ~/OEIS/b188593.txt
  \\ g=OEIS_bfile_gf("A188593");
  \\ rlo = subst(g,        'x,1/10) * 10;
  \\ rhi = subst(g+x^10001,'x,1/10) * 10;
  \\ rlo*1.0
  \\ rhi*1.0
  \\ p = x^4 - 5*x^2 + 5
  \\ subst( p, x, rlo) < 0
  \\ subst( p, x, rhi) > 0
  \\ ploth(x=1,2, subst(p,'x,x))
  \\ polroots(p)

  \\ ~/OEIS/b182007.txt
  \\ g=OEIS_bfile_gf("A182007");
  \\ rlo = subst(g,       'x,1/10) * 10;
  \\ rhi = subst(g+x^1000,'x,1/10) * 10;
  \\ rlo*1.0
  \\ rhi*1.0
  \\ p = x^4 - 5*x^2 + 5
  \\ d = p'
  \\ subst( p, x, rlo) > 0  \\ descending through zero
  \\ subst( p, x, rhi) < 0
  \\ ploth(x=1,2, subst(p,'x,x))

  my(forest=1);
  for(n=1,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           my(p=vpar_adjacency_poly(vpar),
              got=is_roots_close(p));
           \\ print(vpar" "p"  got "got);
           \\ print("  "polrootsreal(p));
           if(!got,
              if(n==-6,
                 print(vpar" "v" poly "p);
                 print("  "polrootsreal(p));
                );
              count++;
             ));
     print1(count","));
  quit;
}

{
  \\ num two-component forests, two roots without integer between,
  \\ but only by a root from each
  \\ 0,0,0,0,0,0,0,2,3,4,7,25,66,151,276,498,
  \\ not in OEIS: 3,4,7,25,66,151,276,498,

  my(forest=1);
  for(n=0,20,
     my(vpar=vpar_free_first(n),
        count=0);
     until((vpar=vpar_free_next_forest(vpar,forest))=='none,
           vpar_num_components(vpar) == 2 || next;
           my(c=vpar_component_trees(vpar));
           if(is_roots_close(vpar_adjacency_poly(c[1]))
              || is_roots_close(vpar_adjacency_poly(c[2])), next);

           my(p=vpar_adjacency_poly(vpar),
              got=is_roots_close(p));
           \\ print(vpar" "p"  got "got);
           \\ print("  "polrootsreal(p));
           if(got,
              if(n==6,
                 print(vpar" "v" poly "p);
                 print("  "polrootsreal(p));
                );
              count++;
             ));
     print1(count","));
  quit;
}



\\------------------------------------------------------------------------------
\\ G and G\x interlace

\\ v is a vector, return 1 if it has some a <= v[i] <= b
has_between(v,a,b) =
{
  for(i=1,#v,
     if(a <= v[i] && v[i] <= b, return(1)));
  0;
}

{
  default(realprecision,6);
  for(n=1,20,
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(spec=real(polroots(vpar_adjacency_poly(vpar))));
           spec=Set(spec);
           spec=vecsort(spec);
           for(v=1,#vpar,
              my(sub=vpar_delete_vertex(vpar,v),
                 subspec=real(polroots(vpar_adjacency_poly(sub))));
              subspec=Set(subspec);
              subspec=vecsort(subspec);
              for(i=1,#subspec-1,
                 if(! has_between(spec, subspec[i],subspec[i+1]),
                    print("   oops");
                 print(vpar" "v" intersect "int);
                 print("  "spec);
                 print("  "subspec);
                   ));
              my(int=setintersect(spec,subspec));
              if(1||#int,
                 ))));
  quit;
}


\\------------------------------------------------------------------------------
\\ Brouwer's tree #17 of 31 vertices
\\ https://hog.grinvin.org/ViewGraphInfo.action?id=34222

{
  depths = [0,1,2, 3,3,3,
            1, 2,3, 2,3, 2,3,
            1, 2,3, 2,3, 2,3,
            1,2,2,2,2,
            1,2,2,2,2,
            1];
}
vpar=vpar_make_preorder_depths(depths);
vpar_validate(vpar);
check_equal(#vpar,31);

check_equal(is_integral_spectrum(vpar), 1);
print(vpar_to_sparse6(vpar));
quit


\\------------------------------------------------------------------------------
\\ Integral Trees

\\ number of n>=1 vertices
\\ not in OEIS: 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 3, 0, 1, 0, 0, 0, 0, 0, 1, 3, 0, 0, 0, 0, 3, 0, 0, 0, 1, 0, 4, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 1, 2
\\ 

\\ Brouwer's table:
\\
{
  v=[1,
     2,
     5,
     6,
     7,
     10,
     14,
     17, 17, 17,
     19,
     25,
     26, 26, 26,
     31, 31, 31,
     35,
     37, 37, 37, 37,
     42,
     46,
     49,
     50, 50 ];
  my(h=vector(50));
  for(i=1,#v, h[v[i]]++);
  print(h);
  quit;
}
                                                              30

\\------------------------------------------------------------------------------
\\ trees with integral spectrum

\\ https://www.win.tue.nl/~aeb/graphs/integral_trees.html

\\          n =
\\ not in OEIS: 1,2,    5,6,7,   10,     14,   17, 19,
\\ not in OEIS: 1,1,0,0,1,1,1,0,0,1,0,0,0,1,0,0,3,0,1,0,

\\ [0]               singleton  x
\\ [0, 1]             path-2   -1,1
\\ [0, 1, 1, 1, 1]     star-5   -2,2
\\ [0, 1, 2, 2, 1, 1]    bistar-3,3  -2,-1,1,2
\\ [0, 1, 2, 1, 4, 1, 6]   subdivided star-4  -2,-1,1,2
\\ [0, 1, 1, 1, 1, 1, 1, 1, 1, 1]   star-10  -3,3
\\ [0, 1, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1]   bistar-7,7  -3,-2,2,3
\\
\\ n=17
\\ [0, 1, 2, 2, 2, 2, 2, 2, 2, 1, 10, 1, 12, 1, 14, 1, 16]
\\    bistar 8,5 with the 5 subdivided to 9
\\ [0, 1, 2, 1, 4, 1, 6, 1, 8, 1, 10, 1, 12, 1, 14, 1, 16]
\\    star 9 subdivided
\\ [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
\\    star 17
\\
\\ n=19    -3,-2,-1,1,2,3
\\    bistar 6,7 with the 7 subdivided to 13
\\ [0, 1, 2, 2, 2, 2, 2, 1, 8, 1, 10, 1, 12, 1, 14, 1, 16, 1, 18]
\\
\\ factor( charpoly(vpar_adjacency_matrix([0, 1, 2, 2, 2, 2, 2, 1, 8, 1, 10, 1, 12, 1, 14, 1, 16, 1, 18])))
\\

try(vpar) =
{
  if(is_integral_spectrum(vpar),
     count++;
     print(vpar);
    );
}
{
  for(n=1,21,
     count=0;
     read(Str("/tmp/trees-"n".gp"));
     \\ print1(count",");
  );
}

{
  for(n=1,20,
     my(vpar=vpar_free_first(n),
         count=0);
     until((vpar=vpar_free_next(vpar))=='none,
           my(is_integral=is_integral_spectrum(vpar));
           count += is_integral;
           if(0 && is_integral && n==8, print(vpar));
          );
     print1(count", ");
     );
  quit;
}

\\-----------------------------------------------------------------------------
\\ integral n=14

vpar = [0, 1, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1, 1];
print("adjacency");
vpar_adjacency_poly(vpar)
factor(vpar_adjacency_poly(vpar))
print("match");
poly=vpar_matchpoly(vpar)
factor(poly)
poly=subst(polrecip(poly),'x,-'x)
factor(poly)
quit


\\------------------------------------------------------------------------------

\\ 10 minutes at 6000 bogomips
\\ 10*6000

\\ f is an Nx2 factor matrix like factor() returns or factorback() takes.
\\ Return 1 if x == factorback(f), by checking and dividing rather than
\\ forming the whole factorback(f).
\\
is_factorback(x,f) =
{
  for(i=1,matsize(f)[1],
     my(t=f[i,1]); \\ term
     for(j=1,f[i,2],
        if(x % t, return(0));
        x /= t));
  x==1;
}

{
  my(S=[1,2,3,4,5,6]);
  my(vpar=make_S(S));
  print("made vpar");
  gettime();
  my(poly=vpar_adjacency_poly(vpar));
  print("made poly, took "gettime()" ms");
  print("log2 biggest coeff "#binary(vecmax(Vec(poly))));
  print("sizebyte poly = "sizebyte(poly));

  check_equal(is_factorback(poly, [x-6, 1;
                                   x-5, 10;
                                   x-4, 89;
                                   x-3, 611;
                                   x-2, 2944;
                                   x-1, 8021;
                                   x, 3655;
                                   x+1, 8021;
                                   x+2, 2944;
                                   x+3, 611;
                                   x+4, 89;
                                   x+5, 10;
                                   x+6, 1]),
              1);
  quit;
}

\\------------------------------------------------------------------------------

\\ 1, 2, 7, 37, 266, 2431, 27007
\\ A001515 Bessel polynomial y_n(x) evaluated at x=1.
\\ A144301 a(0) = a(1) = 1; thereafter a(n) = (2*n-3)*a(n-1) + a(n-2).
\\ 
vector(6,n,n--; #make_S([1..n]))
quit

\\------------------------------------------------------------------------------

make_T([3])
make_T([2,2,2])
#make_S([1,2,4,5])
#make_S([1,2,3,4,5,6])

factor(vpar_adjacency_poly(make_S([1,2,4,5])))
{
  my(S=[1,2,4,5]);
  check_equal(S_to_r(S), [9,12,3,1])

  my(vpar=make_S(S));
  check_equal(#vpar, 781);
  check_equal(vpar_adjacency_poly(vpar),
              (x+5)*(x-5)
              * ( (x+4)*(x-4) )^8
              * ( (x+2)*(x-2) )^100
              * ( (x+1)*(x-1) )^227
              * x^109);
}
{
  my(S=[1,2,3,4,5,6]);
  check_equal(S_to_r(S), [11,9,7,5,3,1])

  my(vpar=make_S(S));
  check_equal(#vpar, 27007);
  check_equal(vpar_diameter(vpar), 12);
  check_equal(vpar_adjacency_poly(vpar),
              (x+6)*(x-6)
              * ( (x+5)*(x-5) )^10
              * ( (x+4)*(x-4) )^89
              * ( (x+3)*(x-3) )^611
              * ( (x+2)*(x-2) )^2944
              * ( (x+1)*(x-1) )^8021
              * x^3655);
}

\\------------------------------------------------------------------------------
