\\ Copyright 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
option_verbose=1;


\\----------------------------------------------------------------------------
\\ primecode rerootings of paths
\\
\\ Ivan Gutman, Aleksandar Ivic, "On Matula Numbers", Discrete Mathematics,
\\ volume 150, 1996, pages 131-142.  Section 2 theorem 1 on two primenest.

\\ minimum primecode of path, root at centre
\\ not in OEIS: 1, 2, 3, 5, 9, 15, 25, 55, 121, 341, 961, 3937, 16129

\\ product of two primenests, primecode of a path of any rooting
\\ not in OEIS: 1, 2, 3, 4, 5, 6, 9, 10, 11, 15, 22, 25, 31, 33

\\ A007097
primenest(n) = my(p=1); for(i=1,n,p=prime(p)); p;
{
  my(want=A007097_samples[1..12]);
  check_equal(vector(#want,n,n--; primenest(n)),
              want,  \\ A007097_samples OFFSET=0
              "A007097 primeth recurrence, primecode of path n");
}
print("primenest ",vector(10,n,n--;primenest(n)))

pathcodes(n) =
{
  my(vpar=vpar_make_path(n));
  Set(vector(n,v, vpar_to_primecode(vpar_reroot(vpar,v))));
}
for(n=1,8,print(pathcodes(n)));

v=vector(13,n, Set(pathcodes(n)))
limit=50
v=concat(v);
v=select(n->n<=limit,v)
v=vecsort(v)

vector(13,n, vecmin(pathcodes(n)))
quit


\\----------------------------------------------------------------------------
\\ A235121 row n all primecodes isomorphic to n, ascending order
\\
\\ Could re-root at one of each orbit

{
  my(want=OEIS_samples("A235121"),
     v=[]);
  for(n=1,#want,
     if(#v>=#want,break);
     my(vpar=vpar_from_primecode(n),
        others=vector(#vpar,v, vpar_to_primecode(vpar_reroot(vpar,v))));
     v=concat(v,Set(others)));
  v=Vec(v,#want);
  check_equal(v,want, "A235121 primecode n tree rerootings, incl self");
  quit;
}


\\------------------------------------------------------------------------------
\\ primecode n to height

\\ fold() new in gp 2.9.x
A109082_compact(n) =
{
  my(v=factor(n)[,1],d=0);
  while(#v,d++; v=fold(setunion,apply(p->factor(primepi(p))[,1]~, v)));
  d;
}
check_equal(vector(1000,n, A109082_compact(n)), \
            vector(1000,n, vpar_height(vpar_from_primecode(n))));

A109082_by_biggest(n) =
{
  \\ v[] is a prime which is a subtree.
  \\ d[] is the depth of the top of that subtree.
  \\ At end-most v[i], factor primepi(v[i]) to primes f[i] of its children.
  \\ For each such prime f[i], insert it into v, or update its d[] depth if
  \\ have now a deeper occurrence.
  \\
  \\ If same (isomorphic) subtrees occur at different depths then this
  \\ approach factors their primecode just once.  Suspect most of the
  \\ time only small common subtrees will occur, so not much saving.
  \\
  my(v=List(factor(n)[,1]),d=List(vector(#v)),ret=0);
  while(#v,
        my(p=v[#v],t=d[#v]+1); listpop(v);listpop(d);
        if(p==2,ret=max(ret,t),
           my(f=factor(primepi(p))[,1]);
           \\ print("v="v" "d" "f);
           for(i=1,#f,
              my(pos=setsearch(v,f[i]));
              if(pos,
                 \\ print("  found at pos="pos);
                 d[pos]=max(d[pos],t);
                 ,
                 \\ print("  insert to pos="pos);
                 pos=setsearch(v,f[i],1);
                 listinsert(v,f[i],pos); listinsert(d,t,pos)))));
  ret;
}
check_equal(vector(1000,n, A109082_by_biggest(n)), \
            vector(1000,n, vpar_height(vpar_from_primecode(n))));

\\ Although A109082_by_biggest() has the algorithmic advantage of not
\\ repeating factor(), in measures slower due to overheads.
\\
\\ Since the primecode is feasible only for small numbers, may as well
\\ prefer the compact code.
{
  my(v=vector(1000,i,random(2^27)));
  gettime(); for(i=1,#v, A109082_compact(v[i]));    print("compact "gettime());
  gettime(); for(i=1,#v, A109082_by_biggest(v[i])); print("biggest "gettime());
}
{
  gettime(); for(n=1,50000, A109082_compact(n));    print("compact "gettime());
  gettime(); for(n=1,50000, A109082_by_biggest(n)); print("biggest "gettime());
}

\\----------------------------------------------------------------------------
\\ vpar_to_primecode() maximum

\\ 1, 2, 4, 8, 19, 67, 331, 2221, 19577, 219613, 3042161, 50728129,
\\ A005518 maximum primecode in n vertices

\\ max 1  [0]
\\ max 2  [0, 1]
\\ max 4  [0, 1, 1]
\\ max 8  [0, 1, 1, 1]
\\ max 19  [0, 1, 2, 2, 2]
\\ max 67  [0, 1, 2, 3, 3, 3]
\\ max 331  [0, 1, 2, 3, 4, 4, 4]
\\ max 2221  [0, 1, 2, 3, 4, 5, 5, 5]
\\ max 19577  [0, 1, 2, 3, 4, 5, 6, 6, 6]
\\ max 219613  [0, 1, 2, 3, 4, 5, 6, 7, 7, 7]
\\ max 3042161  [0, 1, 2, 3, 4, 5, 6, 7, 8, 8, 8]
\\ max 50728129  [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 9, 9]

{
  for(n=1,12,
     my(vpar=vpar_unlabelled_first(n),
        m_vpar=vpar,
        m_primecode=-1);
     until((vpar=vpar_unlabelled_next(vpar))=='none,
           my(primecode=vpar_to_primecode(vpar));
           if(primecode > m_primecode,
              m_primecode = primecode;
              m_vpar = vpar));
     print("max "m_primecode"  "m_vpar);
     \\ print1(m_primecode", ");
     );
  quit;
}


\\-----------------------------------------------------------------------------
