\\ Copyright 2021, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1)
read("../vpar.gp");
read("../vpar-wip.gp");
read("../test-helpers.gp");
read("OEIS-data.gp");
read("OEIS-data-wip.gp");
read("../examples/terminal-Wiener-maximum.gp");

option_verbose=0;
progress_prev=-1;


\\------------------------------------------------------------------------------
\\ Generic

is_integer(x) = x==floor(x);

A004526(n) = floor(n/2);
{
  my(want=OEIS_data("A004526")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A004526(n)), want,
              "A004526()");
}

A004523(n) = n>=0 || error(); floor(2*n/3);
{
  my(want=OEIS_data("A004523")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A004523(n)), want,
              "A004523()");
}

gf2_term(g,n,k) = g += O(x^(n+1));g = polcoeff(g,n,'x);g += O(y^(k+1));polcoeff(g,k,'y);


\\------------------------------------------------------------------------------
\\ A133188 Rows of 3

A133188_row(n) =
{
 if(A004526(n-1)==0, [A004526(n-1), A004526(n), 1],
                     [1, A004526(n), A004526(n-1)]);
}
{
  \\ example table
  my(want=[[0, 0, 1],
           [0, 1, 1],
           [1, 1, 1],
           [1, 2, 1],
           [1, 2, 2],
           [1, 3, 2]]);
  check_equal(vector(#want,n, A133188_row(n)),
              want)
}

A133188_flat(n) =
{
  n+=2;
  my(s=n\3,r=n%3);
  if(A004526(s-1)==0, [A004526(s-1), A004526(s), 1][r+1],
                      [1, A004526(s), A004526(s-1)][r+1]);
}
{
  my(want=OEIS_data("A133188")); \\ OFFSET=1
  check_equal(vector(#want,n, A133188_flat(n)), want,
              "A133188_flat()");

  check_equal(concat(vector(100,n,A133188_row(n))),
              vector(300,n,A133188_flat(n)),
              "A133188_row() vs A133188_flat()");
}


\\------------------------------------------------------------------------------
\\ Partitions into <=3 Parts
\\ A001399

numbpart_le_3(n) = (n+3)^2 \/12;
A001399 = numbpart_le_3;
{
  my(want=OEIS_data("A001399")); \\ OFFSET=0
  check_equal(vector(#want,n,n--; A001399(n)), want,
              "A001399() vs samples");

  check_equal(vector(50,n,n--; A001399(n)),
              vector(50,n,n--; #partitions(n,,3)),
              "A001399() vs #partitions");
  check_equal(vector(10,n,n+=3; A001399(n-4)),
              vector(10,n,n+=3; #partitions(n-1,,[3,3])),
              "A001399() vs #partitions");
}

gA001399(x) = 1 / ((1 - x) * (1 - x^2) * (1 - x^3));
{
  check_equal(test_gf_terms(gA001399(x),100),
              vector(100,n,n--; A001399(n)),
              "gA001399()");
}


\\------------------------------------------------------------------------------
\\ Terminal Wiener Index k Maximum Construction

{
  check_equal(make_starlike([3,4,5]),
              [0, 1,2,3, 1,5,6,7, 1,9,10,11,12]);
  \\           1  2 3 4  5 6 7 8  9
}

{
  \\   7                7
  \\   |                |
  \\   1--2--3--4    1--2--3--4
  \\   |        |    |        |
  \\   5        6    5        6
  check_equal(make_max_TW_nk_vpars(7,3),
              [[0, 1, 1, 1, 4, 5, 6],
               [0, 1, 1, 3, 1, 5, 6],
               [0, 1, 2, 1, 4, 1, 6]]);

  \\ 7,3
  \\        1
  \\      / | \
  \\     2  4  6
  \\     |  |  |
  \\     3  5  7
  \\ [0, 1, 2, 1, 4, 1, 6]
  check_equal(vpar_relabel_preorder(vpar_reroot([0, 1, 2, 3, 1, 5, 1], 5)),
              [0, 1, 2, 3, 4, 2, 1]);
  check_equal(vpar_relabel_preorder(vpar_reroot([0, 1, 2, 3, 1, 5, 5], 5)),
              [0, 1, 2, 3, 4, 1, 1]);
  \\ my(n=7,k=3); ceil((n-k)/2) == 2

  check_equal(make_max_TW_nk_vpars(9,5),
              [[0, 1, 2, 3,  1, 1, 1, 4, 4],
               [0, 1, 2, 3,  1, 1, 2, 4, 4]]);

  check_equal(make_max_TW_nk_vpars(12,5),
              [[0, 1, 2, 3, 4, 5, 6,   1, 1, 1, 7, 7],
               [0, 1, 2, 3, 4, 5, 6,   1, 1, 2, 7, 7],
               [0, 1, 2, 3, 4, 5, 6,   1, 1, 3, 7, 7],
               [0, 1, 2, 3, 4, 5, 6,   1, 1, 4, 7, 7]]);

  \\ Gutman et al figure 3
  foreach([[4, [[2,0,0,0,0,0,0,2] ] ],
           [5, [[3,0,0,0,0,0,2],
                [2,1,0,0,0,0,2],
                [2,0,1,0,0,0,2],
                [2,0,0,1,0,0,2]] ],
           [6, [[3,0,0,0,0,3]] ],
           [7, [[4,0,0,0,3],
                [3,1,0,0,3],
                [3,0,1,0,3]] ],
           [8, [[4,0,0,4]] ],
           [9, [[5,0,4],
                [4,1,4]] ],
           [10, [[5,5]] ],
           [11, [[11]] ] ],
     elem,
     my(k=elem[1],wantvec=elem[2]);
     wantvec=canonical_vpars(apply(vpar_make_caterpillar,wantvec));
     foreach(wantvec,vpar,
        check_equal(#vpar,12);
        check_equal(vpar_num_leaves(vpar),k));

     my(gotvec=canonical_vpars(make_max_TW_nk_vpars(12,k)));
     check_equal(gotvec,wantvec,
                 Str("make_max_TW_nk_vpars() examples k="k));
  );
}

\\-----------------------------------------------------------------------------
\\ A349702 max_TW_nk() Terminal Wiener Index Maximum by n,k

{ my(n=0,k=0);
  check_equal( k*(k-1) + (n-1-k)*floor(k/2)*ceil(k/2),  0);
}
{ my(n=1,k=1);
  check_equal( k*(k-1) + (n-1-k)*floor(k/2)*ceil(k/2),  0);
}

{
  check_equal(max_TW_nk_row(0), [0]);
  check_equal(max_TW_nk_row(1), [0]);
  check_equal(n_good_ks(0), [0]);

  \\ DATA
  if(0,
     my(v=[]);
     for(n=0,13,
        v=concat(v, apply(k->max_TW_nk(n,k), n_good_ks(n))));
     for(i=1,#v, print1(v[i],",");if(i==27||i==50,print()));
     print();
     quit);

  \\ OFFSET=0
  my(want=OEIS_data("A349702"),
    got=[]);
  for(n=0,oo, 
     if(#got>=#want,break);
     got=concat(got,max_TW_nk_row(n)));
  check_equal(got, want,
              "A349702 max_TW_nk() data");
}
{
  if(0,
     system("rm -f /tmp/b349702.txt");
     my(v=[]);
     for(n=0,120,
        v=concat(v,max_TW_nk_row(n)));
     for(i=1,#v,
        my(n=i-1);
        write("/tmp/b349702.txt", Str(n" "v[i])));
     system("ls -l /tmp/b349702.txt");
     quit);

  \\ ~/OEIS/b349702.txt
  if(0,
     my(want=OEIS_data("A349702",'bfile),  \\ OFFSET=0
        got=[]);
     print("~/OEIS/b349702.txt num terms ",#want);
     for(n=0,oo, 
        if(#got>=#want,break);
        got=concat(got,max_TW_nk_row(n)));
     check_equal(got, want,
                 "A349702 max_TW_nk() data"));
}
{
  \\ example picture
  if(0,
     my(max_n=9,
        k_to_width(k) = if(k<=2,1,2));
     print("max_TW_nk(n,k)");
     print1("%e       ");
     for(k=0,max_n-1,
        printf("%*s", k_to_width(k)+2,
               if(k==0,"k=0",k)));
     print();
     for(n=0,max_n,
        printf("%%e   n=%-3s",Str(n,";"));
        for(k=0,max(2,n-1),
           if(nk_is_good(n,k),
              printf(" %*d,", k_to_width(k), max_TW_nk(n,k)),
              printf(" %*s ", k_to_width(k), "")));
        print());
     quit);
}

{
  my( T(n,k) = (((n-k+3)*k - 4)*k + if(k%2,k-n+1))>>2;
     );
  A349702_max_TW_nk_compact = T;
}
{
  check_equal(matrix(30,30,n,k,n--;k--;
                     if(nk_is_good(n,k), max_TW_nk(n,k), 0)),
              matrix(30,30,n,k,n--;k--;
                     if(nk_is_good(n,k), A349702_max_TW_nk_compact(n,k), 0)),
              "A349702_max_TW_nk_compact()");
}

max_TW_nk_formula(n,k) = local(nk_validate=(n,k)->); max_TW_nk(n,k);
{
  check_equal(max_TW_nk(2,2), 1, "max_TW_nk() 2,2");
  check_equal(max_TW_nk_formula('n,2), 'n-1);
  check_equal(max_TW_nk_formula(n,3), 2*(n-1),
              "max_TW_nk_formula() n,3");

  check_equal(matrix(30,30,n,k,n--;k--; max_TW_nk_formula(n,k)),
              matrix(30,30,n,k,n--;k--; my(o=k%2);
                     ( -k^3 + (n+3)*k^2 + (-4+o)*k - (n-1)*o )/4 ),
              "max_TW_nk_formula() with o");

  check_equal(vector(30,n,n--; floor(k/2)*ceil(k/2)),
              vector(30,n,n--; if(k%2==0, floor(k/2)*(floor(k/2)),
                                          floor(k/2)*(floor(k/2) + 1))));
  check_equal(vector(30,n,n--; floor(k/2)*ceil(k/2)),
              vector(30,n,n--; if(k%2==0, k^2/4,
                                          k^2/4 + (k-1)/2)));


  my(even = subst( -k^3 + (4+n-1)*k^2 + (o^2 - 4)*k - (n-1)*o^2, o, 0));
  check_equal(even, -k*( k^2 - (n+3)*k + 4 ));
  my(evenf(k) = -k*( k^2 - (n+3)*k + 4 ));
  \\ my(n=100); plot(k=0,100, -k*( k^2 - (n+3)*k + 4 ))
  check_equal(even,
              -k*( (k-(n+3)/2)^2 - (n-1)*(n+7)/4));
  my(deven = deriv(even,k));
  check_equal(deven, -3*k^2 + 2*(n+3)*k - 4 );
  check_equal(deven, -3*k^2 + (2*n+6)*k - 4);
  my(devenf(k) = -3*k^2 + (2*n+6)*k - 4);
  check_equal(deven,
              -3*( (k - (1/3*n+1))^2  - ((2*n+6)/6)^2 + 4/3));
  \\ my(n=100); plot(k=0,100, -3*k^2 + 2*(n+3)*k - 4)
  check_equal(devenf(n/3), 1/3*n^2 + 2*n - 4);
  check_equal(devenf(2*n/3), 4*n - 4);

  my(odd = subst( -k^3 + (4+n-1)*k^2 + (o^2 - 4)*k - (n-1)*o^2, o, 1));
  check_equal(odd,
              -k^3 + (4+n-1)*k^2 + (1 - 4)*k - (n-1),
              "odd");
  check_equal(odd,
              -k*( (k-(n+3)/2)^2 - (n-1)*(n+7)/4 - 1)  - (n-1),
              "odd again");
  my(dodd = deriv(odd,k));
  check_equal(dodd,
              -3*k^2 + (2*n+6)*k - 3,
              "dodd derivative");
}

check_equal(max_TW_nk_row(6), [5, 10, 16, 20]);
check_equal(max_TW_nk_row(2), [1]);
\\
\\ vector(15,n,n+=2; vecsum(max_TW_nk_row(n)))
\\ not in OEIS: 2, 9, 24, 51, 94, 158, 248, 370, 530, 735, 992, 1309, 1694, 2156, 2704

max_TW_nk_rowpoly(n,var=y) =
{
  n>=0 || error();
  my(v=n_good_ks(n));
  sum(i=1,#v, my(k=v[i]);
      max_TW_nk(n,k) * var^k);
}
check_equal(max_TW_nk_rowpoly(0), Pol(0,y));
check_equal(max_TW_nk_rowpoly(2), 1*y^2);
check_equal(max_TW_nk_rowpoly(6), 5*y^2 + 10*y^3 + 16*y^4 + 20*y^5);

\\ recurrence_guess(vector(500,n,n--; if(n>=3,max_TW_nk_rowpoly(n,y)))); quit;

gmax_TW_nk(x,y) =
{
  x^2*y^2*( 1 + x*(1 + (1-x)*(1+2*x*y)) / ((1-x)^2 * (1+x*y) * (1-x*y)^3) )
  ;
}
\\ matrix(5,10,n,k,n--;k--; gf2_term(gmax_TW_nk(x,y),n,k))
\\ matrix(5,10,n,k,n--;k--; iferr(max_TW_nk(n,k),e,'e))

{
  check_equal(subst((2 + (-1+2*y)*x - 2*y*x^2),x,z), (2-z) + 2*(1-z)*z*y);
  check_equal(subst((2 + (-1+2*y)*x - 2*y*x^2),x,z), 1 + (1-z)*(1+2*z*y));
}

{
  check_equal(matrix(30,30,n,k,n--;k--; gf2_term(gmax_TW_nk(x,y), n,k)),
              matrix(30,30,n,k,n--;k--; if(nk_is_good(n,k), max_TW_nk(n,k), 0)),
              "gmax_TW_nk() vs max_TW_nk() 30");

  my(limit=100,
     g=gmax_TW_nk(x,y) + O(x^limit) + O(y^limit));
  check_equal(matrix(limit,limit,n,k,n--;k--; polcoeff(polcoeff(g,n,'x), k,'y)),
              matrix(limit,limit,n,k,n--;k--; if(nk_is_good(n,k), max_TW_nk(n,k), 0)),
              "gmax_TW_nk() vs max_TW_nk() 100");
}

\\------------------------------------------------------------------------------
\\ A349703 countmax_TW_nk()   n,k count
\\
\\ k even, k/2 on each end, unique
\\ k odd,

{
  my(k=3);
  for(n=4,50, 
     countmax_TW_nk(n,k) == A001399(n-4) || error();
     countmax_TW_nk(n,k) == round((n-1)^2 / 12)  || error());
}
                
{
  my( T(n,k) = if(n==1||k%2==0,1, k==3,(n-1)^2\/12, (n-k+1)>>1);
     );
  A349703_countmax_TW_nk_compact = T;
  check_equal(T(0,0), 1);
  check_equal(T(1,1), 1);
  check_equal(T(2,2), 1);
}
{
  check_equal(matrix(30,30,n,k,n--;k--;
                     if(nk_is_good(n,k), countmax_TW_nk(n,k), 0)),
              matrix(30,30,n,k,n--;k--;
                     if(nk_is_good(n,k), A349703_countmax_TW_nk_compact(n,k), 0)),
              "A349703_countmax_TW_nk_compact()");
}


countmax_TW_nk_row(n) =
{
  n>=0 || error();
  apply(k->countmax_TW_nk(n,k), n_good_ks(n));
}
check_equal(countmax_TW_nk_row(6), [1,2,1,1]);
check_equal(countmax_TW_nk_row(2), [1]);

{
  check_equal(countmax_TW_nk_row(0), [1]);
  check_equal(countmax_TW_nk_row(1), [1]);

  \\ DATA
  if(0,
     my(v=[]);
     for(n=0,15, v=concat(v, countmax_TW_nk_row(n)));
     for(i=1,#v, print1(v[i],",");if(i==32||i==63,print()));
     print();
     quit);

  my(want=OEIS_data("A349703"),  \\ OFFSET=0
    got=[]);
  for(n=0,oo, 
     if(#got>=#want,break);
     got=concat(got,countmax_TW_nk_row(n)));
  check_equal(got, want,
              "A349703 countmax_TW_nk() data");
}
{
  if(0,
     system("rm -f /tmp/b349703.txt");
     my(v=[]);
     for(n=0,140,
        v=concat(v,countmax_TW_nk_row(n)));
     for(i=1,#v,
        my(n=i-1);
        write("/tmp/b349703.txt", Str(n" "v[i])));
     system("ls -l /tmp/b349703.txt");
     quit);

  \\ ~/OEIS/b349703.txt
  if(0,
     my(want=OEIS_data("A349703",'bfile),  \\ OFFSET=0
        got=[]);
     print("~/OEIS/b349703.txt num terms ",#want);
     for(n=0,oo, 
        if(#got>=#want,break);
        got=concat(got,countmax_TW_nk_row(n)));
     check_equal(got, want,
                 "A349703 countmax_TW_nk() data"));
}
{
  \\ example picture
  if(0,
     my(max_n=9,
        k_to_width(k) = 1);
     print("countmax_TW_nk(n,k)");
     print1("%e       ");
     for(k=0,max_n-1,
        printf("%*s", k_to_width(k)+2,
               if(k==0,"k=0",k)));
     print();
     for(n=0,max_n,
        printf("%%e   n=%-3s",Str(n,";"));
        for(k=0,max(2,n-1),
           if(nk_is_good(n,k),
              printf(" %*d,", k_to_width(k), countmax_TW_nk(n,k)),
              printf(" %*s ", k_to_width(k), "")));
        print());
     quit);
}

\\ general case part of countmax_TW_nk()
countmax_TW_nk_formula(n,k) = if(k%2==0, 1, ceil((n-k)/2));

{
  check_equal(countmax_TW_nk_formula(0,0), 1);
  check_equal(countmax_TW_nk_formula(1,1), 0); \\ but want 1

  check_equal(countmax_TW_nk(2,2),
              #make_max_TW_nk_vpars(2,2),
              "countmax_TW_nk() 2,2");
  check_equal(vector(20,n,n++; countmax_TW_nk(n,2)),
              vector(20,n,n++; 1),
              "countmax_TW_nk() n,2");

  check_equal(countmax_TW_nk(7,3), 3,
              "countmax_TW_nk(7,3)");

  for(n=2,20,
     for(k=2,max(2,n-1),
        check_equal(countmax_TW_nk(n,k),
                    #make_max_TW_nk_vpars(n,k),
                    "countmax_TW_nk() vs #make_max_TW_nk_vpars()")));
}
{
  \\ example picture
  if(0,
     print("countmax_TW_nk(n,k)");
     print1("       ");
     for(k=2,10,
        printf(" k=%d",k));
     print();
     for(n=2,11,
        printf("  n=%-3s",Str(n,";"));
        for(k=2,max(2,n-1),
           printf(" %2d,", countmax_TW_nk(n,k)));
        print());
     quit);
}
\\ matrix(12,10,n,k,n--;k--; iferr(countmax_TW_nk(n,k),e,'e))

countmax_TW_nk_rowpoly(n,x='x) =
{
  n>=0 || error();
  if(n<=1, 0*x, Polrev(countmax_TW_nk_row(n),x)*x^2);
}

gcount_k3(x) = x^4 / ((1 - x) * (1 - x^2) * (1 - x^3));
{
  check_equal(vector(100,n,n--; if(nk_is_good(n,3), countmax_TW_nk(n,3), 0)),
              test_gf_terms(gcount_k3(x),100),
              "gcount_k3()");
  check_equal(gcount_k3(x), x^4 * gA001399(x));
}

gcountmax_TW_nk(x,y) =
{
  1 + x*y + x^2*y^2 + x^3*y^2*(1 - x^2 - x^3 + x^5 + x*y - x^6*y^3) / ((1-x)^2 * (1-x^2) * (1-x*y) * (1+y*x) * (1+x+x^2))
  ;
}
{
  check_equal(matrix(40,40,n,k,n--;k--; gf2_term(gcountmax_TW_nk(x,y), n,k)),
              matrix(40,40,n,k,n--;k--; if(nk_is_good(n,k), countmax_TW_nk(n,k), 0)),
              "gcountmax_TW_nk() vs max_TW_nk() 40");

  \\ separate k<=3
  check_equal(1 + x*y                                              \\ n=0,1
              + x^2*y^2               /  (1-x)                     \\ k=2 column
              + x^4*y^3               / ((1-x)*(1-x^2) * (1-x^3))  \\ k=3 column
              + x^5*y^4 * (1+x*y-x^2) / ((1-x)*(1-x^2) * (1-x^2*y^2) )  \\ for k>=4
              ,
              gcountmax_TW_nk(x,y));

  \\ separate k<=3 common factors
  check_equal(1 + x*y                               \\ n=0,1
              + 1/(1-x) * (x^2*y^2                  \\ k=2 column
                           + 1/(1-x^2) * (
                                          x^4*y^3               / (1-x^3)   \\ k=3 column
                                          + x^5*y^4 * (1+x*y-x^2) / (1-x^2*y^2)  \\ for k>=4
                           )
                          )
              ==
              gcountmax_TW_nk(x,y),
              1);
  \\ compact:
  check_equal(
  1 + x*y + ( x^2*y^2 + ( x^4*y^3/(1-x^3) + x^5*y^4*(1+x*y-x^2)/(1-x^2*y^2) )/(1-x^2) )/(1-x)
              ==
              gcountmax_TW_nk(x,y),
              1);
  \\ with denominators at front
  check_equal(
  1 + x*y + (1/(1-x))*( x^2*y^2 + (1/(1-x^2))*( x^4*y^3/(1-x^3) + x^5*y^4*(1+x*y-x^2)/(1-x^2*y^2) ) )
              ==
              gcountmax_TW_nk(x,y),
              1);

  \\ separate k=3 and n=0..2
  check_equal(1 + x*y + x^2*y^2                         \\ n=0,1,2
              + x^4*y^3 / ((1-x) * (1-x^2) * (1-x^3))   \\ k=3 column
              + x^3 * (y)^2 * (1 - x^2 + y^3*x^3) / ( (1-x) * (1-x^2) * (1-x^2*y^2) )
              ==
              gcountmax_TW_nk(x,y),
             1);

  \\ separate k=3 and n=0,1
  check_equal(1 + x*y
              + x^4*y^3 / ((1 - x) * (1 - x^2) * (1 - x^3))   \\ k=3 column
              + x^2*y^2 * (1 - (1 + y^2)*x^2 + x^3*y^2 + (y^2 + y^3)*x^4 - y^2*x^5)
              / ( (1-x) * (1-x^2) * (1-x^2*y^2) )
              ==
              gcountmax_TW_nk(x,y),
              1);
}
{
  if(0,
     recurrence_guess(vector(500,n,n--;
                             sum(k=0,n,
                                 if(nk_is_good(n,k) && n>=2 && k>=5 && k%2==1,
                                    countmax_TW_nk(n,k)*y^k))));
     quit);

  \\ even k>=2 columns only
  \\ x^2*y^2*(1 - x^2*y^2*(1+x)) / ((1-x) * (1-x^2*y^2))

  \\ odd k>=5 columns only
  \\ x^6*y^5 / ( (1-x)*(1-x^2) * (1-x^2*y^2) )
}

{
  check_equal(subst((1 - x^2 - x^3 + x^5 + x*y - y^3*x^6),'x,'z),
              -z^6*y^3 + z*y + (z^5 - z^3 - z^2 + 1));

  check_equal(
              -1 * (y)^2 * x^3 * (-1 - y*x + x^2 + x^3 - x^5 + y^3*x^6) / ( (1 - x)^3 * (1 + x) * (1 - y*x) * (1 + y*x) * (1 + x + x^2) )
              ==
  (-y^3 + 1) - y^3*x
  + (-143*y^7 + 232*y^5 + 72*y^4 - 17*y^3 - 72*y^2)/(-72*y^4 + 144*y^2 - 72)/(1 - x)
  + (-11*y^5 + 5*y^3)/(12*y^2 - 12)/(1 - 2*x + x^2)
  + 1/6*y^3/(1 - x)^3
  + (y^5 + y^3)/(-8*y^2 + 8)/(1 + x)
  + (2*y^2 - 1)/(2*y^3 - 2*y^2 - 2*y + 2)/(1 - y*x)
  + 1/(2*y^3 + 2*y^2 - 2*y - 2)/(1 + y*x)
  + (-1/9*y^3 + 1/9*y^3*x)/(1 + x + x^2),
              1);

}
\\------------------------------------------------------------------------------
\\ Terminal Wiener Index Maximum Construction

make_max_TW_all(n) =
{
  n>=0 || error();
  my(s=n\3);
  if(n<=9,
     [vpar_make_star(n)];
     ,
     n%3==0,
     make_max_TW_nk_vpars(n,2*s+2);
     ,
     n%3==1,
     concat(make_max_TW_nk_vpars(n,2*s+2),
            make_max_TW_nk_vpars(n,2*s+3));
     ,
     n%3==2,
     make_max_TW_nk_vpars(n,2*s+3);
     ,
     error("oops"));
}
{
  check_equal(make_max_TW_all(0), [ [] ],
              "make_max_TW_all() 0");

  check_equal(make_max_TW_all(8), [[0, 1, 1, 1, 1, 1, 1, 1]],
              "make_max_TW_all() 8 = star only");

  \\      5 leaves * -- * 4 leaves
  check_equal(make_max_TW_all(11), [ [0, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2] ],
              "make_max_TW_all() 11");
  check_equal(2*3+2, 8);
  check_equal(vpar_is_free_isomorphic(make_max_TW_all(11)[1],
                                      vpar_make_bistar(6,5)), 1,
              "make_max_TW_all() 11 = bistar");
}


\\------------------------------------------------------------------------------
\\ max_TW() Terminal Wiener Index Maximum

{
  my(a=max_TW);
  for(n=15,100, a(n) == 2*a(n-1) - a(n-2) + 2*a(n-3) - 4*a(n-4) + 2*a(n-5) - a(n-6) + 2*a(n-7) - a(n-8)
                       || error());
  for(n=3,9, a(n) == (n-1)*(n-2) || error(n));

  check_equal(a(2),1);
  check_equal(subst((n-1)*(n-2),n,2), 0);;
}
{
  \\ A744443 compact
  my(a(n) = if(n<4, max(0,n-1), n<7,(n-1)*(n-2), (((n+9)*n + if(n%3,6,9))*n - 1)\27);
    );
  check_equal(vector(1000,n,n--; a(n)),
              vector(1000,n,n--; max_TW(n)),
              "a(n) compact");
}
{
  \\ DATA
  if(0,
     for(n=0,50, print1(max_TW(n),",");if(n==21||n==36,print()));
     print();
     quit);

  my(want=OEIS_data("A349704"));  \\ OFFSET=0
  check_equal(vector(#want,n,n--; max_TW(n)), want,
              "A349704 max_TW() data");
}
{
  if(0,
     system("rm -f /tmp/b349704.txt");
     my(v=[]);
     for(n=0,5000,
        write("/tmp/b349704.txt", Str(n" "max_TW(n))));
     system("ls -l /tmp/b349704.txt");
     quit);

  \\ ~/OEIS/b349704.txt
  if(0,
     my(want=OEIS_data("A349704",'bfile));  \\ OFFSET=0
     print("~/OEIS/b349704.txt num terms ",#want);
     check_equal(vector(#want,n,n--; max_TW(n)), want,
                 "A349704 max_TW() b-file data"));
}

\\ general case only, not stars n<=9
max_TW_formula(n) =
{
  my(s=n\3);
  if(n%3==0, s^3 + 3*s^2 + s - 1,
     n%3==1, s^3 + 4*s^2 + 3*s,
     n%3==2, s^3 + 5*s^2 + 6*s + 2);
}
\\ recurrence_guess(vector(100,n,n--;max_TW_formula(n))); quit;
\\   1/27 * n^3
\\   + 1/3 * n^2
\\   + vector_modulo([1/3, 2/9, 2/9],n) * n
\\   + vector_modulo([-1, -16/27, -2/27],n)
\\ factor(n^2+9*n + 9)
\\ factor(n^2+9*n + 81/4)
{
  check_equal(vector(50,n,n+=9; max_TW_formula(n)),
              vector(50,n,n+=9; max_TW(n)));

  check_equal((2*n+9)^2, 4*(n^2 + 9*n + 81/4));
  check_equal(vector(50,n,n--; max_TW_formula(n)),
              vector(50,n,n--;
                     (n^3
                     + 9*n^2
                     + test_vector_modulo([9, 6, 6],n) * n
                     - test_vector_modulo([27, 16, 2],n))/27
                     ));
  check_equal(vector(50,n,n--; max_TW_formula(n)),
              vector(50,n,n--;
                     (((2*n+9)^2 - if(n%3,57,45))*n - 1)\108
                     ));

  check_equal(vector(50,n,n--; max_TW_formula(n)),
              vector(50,n,n--;
                     (((n+9)*n + if(n%3,6,9))*n - 1)\27
                     ));
}

\\ recurrence_guess(vector(100,n,n--;max_TW(n))); quit;
{
  \\ max_TW_formula() versus actual (n-1)*(n-2)
  check_equal(vector(20,n,n--; max_TW(n) == max_TW_formula(n)),
              vector(20,n,n--; n==1 || n>=7),
              "where max_TW = max_TW_formula");
  check_equal(vector(20,n,n--; max_TW(n) == (n-1)*(n-2)),
              vector(20,n,n--; n==1 || (3 <= n && n <= 10)),
              "where max_TW = star (n-1)*(n-2)");

  check_equal(vector(11,n,n--; max_TW(n)),
              [ 0, 0, 1, 2, 6, 12, 20, 30, 42, 56,  72]);
  check_equal(vector(11,n,n--; max_TW_formula(n)),
              [-1, 0, 2, 4, 8, 14, 21, 30, 42, 56,  72]);
  check_equal(vector(11,n,n--; (n-1)*(n-2)),
              [ 2, 0, 0, 2, 6, 12, 20, 30, 42, 56,  72]);
  \\            0  1  2  3  4   5   6

  check_equal(vector(11,n,n--; max_TW(n) - max_TW_formula(n)),
              [1, 0, -1,  -2, -2, -2, -1,  0, 0, 0, 0]);

  check_equal(max_TW(11), 92);
  check_equal(max_TW(11), 3*5*4 + 2*binomial(5,2) + 2*binomial(4,2));
}


gmax_TW_formula(x) = (-1 + 2*x + x^2 + 2*x^3 - 2*x^4) / ((1-x)^4*(1+x+x^2)^2);
{
  check_equal(vector(100,n,n--; max_TW_formula(n)),
              test_gf_terms(gmax_TW_formula(x),100),
              "gmax_TW_formula() vs max_TW_formula()");
  check_equal(test_gf_term(gmax_TW_formula(x),0), -1);

  my(g1=(gmax_TW_formula(x)+1)/x);
  check_equal(g1, x * (1 + x^2) * (2 - 2*x^3 + x^4)
                  / ( (1 - x)^4 * (1 + x + x^2)^2 ));

  check_equal(test_gf_term(g1,0), 0);
  my(g2=g1/x);
  check_equal(g2, (1 + x^2) * (2 - 2*x^3 + x^4)
                  / ( (1 - x)^4 * (1 + x + x^2)^2 ),
             "g2");

  check_equal(test_gf_term(g2,0), 2);
  my(g3=(g2-2)/x);
  check_equal(g3, -1 * (-4 - 2*x^2 + 7*x^3 - 2*x^4 + x^5 - 4*x^6 + 2*x^7)
                  / ( (1 - x)^4 * (1 + x + x^2)^2 ),
              "g3");
}

gmax_TW(x) =
{
  1 - x^2 - 2*x^3 - 2*x^4 - 2*x^5 - x^6 + (-1 + 2*x + x^2 + 2*x^3 - 2*x^4) / ((1-x)^4*(1+x+x^2)^2)
;
}
{
  check_equal(test_gf_terms(gmax_TW(x),100),
              vector(100,n,n--; max_TW(n)),
              "gmax_TW() vs max_TW()");
  check_equal(polisirreducible((-1 + 2*x + x^2 + 2*x^3 - 2*x^4)), 1);
}


\\-----------------------------------------------------------------------------
\\ A133188 Repetitions

\\ Gutman et al figure 4 trees
check_equal(countmax_TW(9), 1);
check_equal(countmax_TW(10), 2);
check_equal(countmax_TW(11), 1);
check_equal(countmax_TW(12), 1);
check_equal(countmax_TW(13), 2);
check_equal(countmax_TW(14), 2);
check_equal(countmax_TW(15), 1);
check_equal(countmax_TW(16), 3);

\\ general case only from countmax_TW()
countmax_TW_formula(n) =
{
  my(s=n\3);
  if(n%3==0, 1,
     n%3==1, ceil(s/2),
     n%3==2, ceil((s-1)/2));
}

{
  check_equal(apply(countmax_TW, [0..3]),
              [1, 1, 1, 1],
              "countmax_TW() initial values");

  \\ for n>=4
  check_equal(vector(50,n,n+=3; countmax_TW(n)),
              vector(50,n,n+=3; A133188_flat(n+1)));

  check_equal(vector(100,n,n--; countmax_TW(n)),
              vector(100,n,n--; #make_max_TW_all(n)));

  check_equal(apply(countmax_TW,[0..9]),
              vector(10,n,n--; 1));
  check_equal(countmax_TW(10), 2,
              "countmax_TW(10)");
}

\\------------------------------------------------------------------------------
\\ Terminal Wiener Index Maximum at which k

\\ only the general case part
max_TW_at_ks_formula(n) =
{
  my(s=n\3);
  if(n%3==0, [ 2*s+2        ],
     n%3==1, [ 2*s+2, 2*s+3 ],
     n%3==2, [        2*s+3 ]);
}

{
  check_equal(max_TW_at_ks(9), [8]);
  check_equal(max_TW_at_ks_formula(9), [8]);

  check_equal(vector(100,n,n--; max_TW_at_ks(n) == max_TW_at_ks_formula(n)),
              vector(100,n,n--;  n>=8 ),
              "max_TW_at_ks() equal to max_TW_at_ks_formula()");

  check_equal(vector(20,n,n++; max_TW_at_ks(n)),
              vector(20,n,n++;
                     my(row=max_TW_nk_row(n),
                        max_tw=max_TW(n),
                        ksub1s=select(tw->tw==max_tw,row,1));
                     apply(ksub1->ksub1+1,Vec(ksub1s))));

  check_equal(vector(20,n,n+=8; max_TW_at_ks(n)),
              vector(20,n,n+=8;
                     select(is_integer,[2*n/3+2,
                                        2*n/3+2 - 2/3,
                                        2*n/3+2 + 1/3,
                                        2*n/3+2 - 1/3])));

  check_equal(vector(20,n,n+=7; max_TW_at_ks(n)),      \\ n>=8
              vector(20,n,n+=7;
                     my(kmax=A004523(n)+2);
                     if(n%3==1,[kmax,kmax+1],[kmax])));

  check_equal(vector(100,n,n--; A004523(n)+2),
              vector(100,n,n--; A004523(n+3)),
              "A004523 indexing");
  \\ vector(30,n,n--; floor(2*n/3+2))
  \\                2,2,3,4,4,5,6,6,7,8,8
  \\ A004523 ,0,0,1,2,2,3,4,4,5,6,6,7,8,8,9,10,10
}
{
  if(0,
     my(min_n=18,
        max_n=min_n+2,
        min_k=12);
     print("max_TW_nk(n,k)");
     print1("%e       ");
     for(k=min_k,max_n-1,
        printf("  k=%d",k));
     print();
     for(n=min_n,max_n,
        my(m=max_TW_at_ks(n));
        printf("%%e   n=%-3s",Str(n,";"));
        for(k=min_k,max(2,n-1),
           my(star=if(setsearch(m,k,),"*"," "));
           printf(" %s%3d,",star,max_TW_nk(n,k)));
        print());
     quit);
}

\\-----------------------------------------------------------------------------
\\ Search
{
\\ [0, 1, 2, 6, 12, 20, 30, 42, 56, 72, 92, 115, 140, 170, 204, 240, 282, 329]
\\ [1, 1, 2, 2, 2, 2, 2, 2, 2, 4, 4, 2, 7, 9, 3, 12, 13, 3]

  for(n=0,12,
     my(m=-1,
        m_vpars=List([]),
        by_k=Map(),
        by_k_vpars=Map());
     my(vpar=vpar_free_first(n));
     until((vpar=vpar_free_next(vpar))=='none,
           my(tw=vpar_Wiener_index(vpar,1),
              k=vpar_num_leaves(vpar));
           if(tw==m, listput(m_vpars,vpar),
              tw>m, m=tw; m_vpars=List([vpar]));

           my(prev=-1,prev_vpars=[]);
           mapisdefined(by_k,k,&prev);
           mapisdefined(by_k_vpars,k,&prev_vpars);
           if(tw>prev,
              mapput(by_k,k,tw);
              mapput(by_k_vpars,k,[vpar]);
              ,
              tw==prev,
              mapput(by_k_vpars,k,concat(prev_vpars,[vpar])));
          );
     check_equal(m, max_TW(n), Str("max_TW() by search ",n));

     my(reps=#m_vpars);
     check_equal(reps, countmax_TW(n), "countmax_TW() by search");

     m_vpars=canonical_vpars(Vec(m_vpars));
     my(by_make=canonical_vpars(make_max_TW_all(n)));
     check_equal(m_vpars,by_make, "make_max_TW_all() vs search");

     by_k=Mat(by_k);
     \\ print("n="n" "by_k);
     for(i=1,matsize(by_k)[1],
        my(k=by_k[i,1]);
        check_equal(max_TW_nk(n,k),
                    by_k[i,2],
                    Str("max_TW_nk() by search n=",n," k=",k));
        my(vpars=canonical_vpars(mapget(by_k_vpars,k)),
           reps=#vpars);
        my(by_make=canonical_vpars(make_max_TW_nk_vpars(n,k)));
        check_equal(by_make,
                    vpars,
                    "make_max_TW_nk_vpars() vs search");
        check_equal(countmax_TW_nk(n,k),
                    reps,
                    "countmax_TW_nk() vs search");
     );
  );
}

\\-----------------------------------------------------------------------------
check_equal(a,'a);
check_equal(k,'k);
check_equal(n,'n);
check_equal(T,'T);
check_equal(vpar,'vpar);
check_equal(vpars,'vpars);

print("end");
\\ vector(18,n,a_formula(n))
\\ vector(18,n,a(n))
\\ vector(18,n,reps_formula(n))
\\ vector(50,n,reps(n))
