\\ Copyright 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0)
default(strictargs,1);
read("../vpar.gp");


\\ Usage: gp <unlabelled-iteration.gp
\\
\\ This is a simple complete program iterating vpar unlabelled tree
\\ representatives with vpar_unlabelled_next().
\\
\\ The loop is the style presented in the addhelp doc string.  Remember an
\\ until() loop runs the body first and then does the test (even though it's
\\ written test first and body second).  So the initial
\\
\\     vpar = vpar_unlabelled_first()
\\
\\ has the body run, then step
\\ 
\\     vpar = vpar_unlabelled_next(vpar)
\\
\\ and the body runs again etc, with stop on symbol 'none meaning there no
\\ next.
\\

n=5;
print("unlabelled tree representatives, n = "n" vertices");
{
  my(count = 0,
     vpar=vpar_unlabelled_first(n));
  until((vpar=vpar_unlabelled_next(vpar)) == 'none,
        print("  vpar "vpar"    depths "vpar_depths_vector(vpar));
        count++);

  print("  "count" trees, which is vpar_count_unlabelled(n) = "
        vpar_count_unlabelled(n));
  count==vpar_count_unlabelled(n) || error();
  print();
}

\\ New for GP 2.13 and up, the call-by-reference vpar_unlabelled_nextR()
\\ makes an in-place modification to vpar.
\\
\\ This means the vpar vector is not copied each time for the return.  For
\\ small trees, the time to copy is small compared to the various overheads
\\ of the GP interpreter, but an in-place modification is per Beyer and
\\ Hedetniemi and is how they achieve constant amortized time.  (Whereas a
\\ vector copy is linear time in number of vertices.)

{
  my(have_call_by_reference = iferr(eval("(~x)->0");1,e,0));
  if(!have_call_by_reference,
     print("no call-by-reference in this GP it seems, version()=",version());
     quit);
}

print("the same, using call-by-reference");
{
  my(vpar=vpar_unlabelled_first(n));
  until(!vpar_unlabelled_nextR(~vpar),
        print("  vpar "vpar);
        count++);
}
